@@ -17,8 +17,6 @@ import actipy
 from stepcount import __model_version__
 from stepcount import __model_md5__
 
-MODEL_PATH = pathlib.Path(__file__).parent / f"{__model_version__}.joblib.lzma"
-
 
 def main():
 
@@ -28,15 +26,24 @@ def main():
     )
     parser.add_argument("filepath", help="Enter file to be processed")
     parser.add_argument("--outdir", "-o", help="Enter folder location to save output files", default="outputs/")
-    parser.add_argument("--model_path", "-m", help="Enter custom model file to use", default=None)
+    parser.add_argument("--model-path", "-m", help="Enter custom model file to use", default=None)
     parser.add_argument("--force-download", action="store_true", help="Force download of model file")
+    parser.add_argument('--model-type', '-t',
+                        help='Enter model type to run (Self-Supervised Learning model or Random Forest)',
+                        choices=['ssl', 'rf'], default='ssl')
+    parser.add_argument("--pytorch-device", "-d", help="Pytorch device to use, e.g.: 'cpu' or 'cuda:0' (for SSL only)",
+                        type=str, default='cpu')
     args = parser.parse_args()
 
     # Timing
     start = time.time()
 
     # Load file
-    data, info = read(args.filepath)
+    if args.model_type == 'ssl':
+        resample_hz = 30
+    else:
+        resample_hz = 'uniform'
+    data, info = read(args.filepath, resample_hz)
 
     # Output paths
     basename = resolve_path(args.filepath)[1]
@@ -44,11 +51,17 @@ def main():
     os.makedirs(outdir, exist_ok=True)
 
     # Run model
+    model_path = pathlib.Path(__file__).parent / f"{__model_version__[args.model_type]}.joblib.lzma"
+    check_md5 = args.model_path is None
+    model = load_model(args.model_path or model_path, args.model_type, check_md5, args.force_download)
     print("Running step counter...")
-    model = load_model(args.model_path or MODEL_PATH, args.force_download)
     # TODO: implement reset_sample_rate()
-    model.sample_rate = info['SampleRate']
-    model.wd.sample_rate = info['SampleRate']
+    model.sample_rate = info['ResampleRate']
+    model.window_len = int(np.ceil(info['ResampleRate'] * model.window_sec))
+    model.wd.sample_rate = info['ResampleRate']
+
+    model.wd.device = args.pytorch_device
+
     Y = model.predict_from_frame(data)
 
     # Save raw output timeseries
@@ -58,16 +71,17 @@ def main():
     summary = summarize(Y)
     summary['hourly'].to_csv(f"{outdir}/{basename}_HourlySteps.csv")
     summary['daily'].to_csv(f"{outdir}/{basename}_DailySteps.csv")
+    summary['daily_walk'].to_csv(f"{outdir}/{basename}_DailyWalk.csv")
     info['TotalSteps'] = summary['total']
     info['TotalWalking(min)'] = summary['total_walk']
 
     # Impute missing periods & recalculate summary
-    Y = impute_missing(Y)
-    summary = summarize(Y, skipna=False)
-    summary['hourly'].to_csv(f"{outdir}/{basename}_HourlyStepsAdjusted.csv")
-    summary['daily'].to_csv(f"{outdir}/{basename}_DailyStepsAdjusted.csv")
-    info['TotalStepsAdjusted'] = summary['total']
-    info['TotalWalkingAdjusted(min)'] = summary['total_walk']
+    summary_adj = summarize(Y, adjust_estimates=True)
+    summary_adj['hourly'].to_csv(f"{outdir}/{basename}_HourlyStepsAdjusted.csv")
+    summary_adj['daily'].to_csv(f"{outdir}/{basename}_DailyStepsAdjusted.csv")
+    summary_adj['daily_walk'].to_csv(f"{outdir}/{basename}_DailyWalkAdjusted.csv")
+    info['TotalStepsAdjusted'] = summary_adj['total']
+    info['TotalWalkingAdjusted(min)'] = summary_adj['total_walk']
 
     # Save info
     with open(f"{outdir}/{basename}_Info.json", 'w') as f:
@@ -76,15 +90,27 @@ def main():
     # Print
     print("\nSummary\n-------")
     print(json.dumps(info, indent=4, cls=NpEncoder))
-    print("\nEstimated Daily Steps\n---------------------")
-    print(summary['daily'])  # adjusted
+    print("\nEstimated Daily Steps/Walk\n---------------------")
+    print(pd.concat([
+        summary['daily'].rename('StepsCrude'),
+        summary['daily_walk'].rename('WalkCrude(min)'),
+        summary_adj['daily'].rename('StepsAdjusted'),
+        summary_adj['daily_walk'].rename('WalkAdjusted(min)')
+    ], axis=1))
 
     # Timing
     end = time.time()
     print(f"Done! ({round(end - start,2)}s)")
 
 
-def summarize(Y, skipna=True):
+def summarize(Y, adjust_estimates=False):
+
+    if adjust_estimates:
+        Y = impute_missing(Y)
+        skipna = False
+    else:
+        # crude summary ignores missing data
+        skipna = True
 
     def _sum(x):
         x = x.to_numpy()
@@ -98,21 +124,24 @@ def summarize(Y, skipna=True):
     total = np.round(Y.agg(_sum))  # total steps
     hourly = Y.resample('H').agg(_sum).round()  # steps, hourly
     daily = Y.resample('D').agg(_sum).round()  # steps, daily
-    total_walk = (  # total walk (mins)
-        (pd.Timedelta(infer_freq(Y.index)) * Y.mask(~Y.isna(), Y > 0).agg(_sum))
-        .total_seconds() / 60
-    )
+
+    dt = pd.Timedelta(infer_freq(Y.index)).seconds
+    W = Y.mask(~Y.isna(), Y > 0)
+    total_walk = np.round(W.agg(_sum) * dt / 60)
+    daily_walk = (W.resample('D').agg(_sum) * dt / 60).round()
 
     total = nanint(total)
     hourly = pd.to_numeric(hourly, downcast='integer')
     daily = pd.to_numeric(daily, downcast='integer')
-    total_walk = float(total_walk)
+    total_walk = nanint(total_walk)
+    daily_walk = pd.to_numeric(daily_walk, downcast='integer')
 
     return {
         'total': total,
         'hourly': hourly,
         'daily': daily,
         'total_walk': total_walk,
+        'daily_walk': daily_walk,
     }
 
 
@@ -125,7 +154,7 @@ def impute_missing(data: pd.DataFrame, extrapolate=True):
                 data.index[0].floor('D'),
                 data.index[-1].ceil('D'),
                 freq=to_offset(infer_freq(data.index)),
-                closed='left',
+                inclusive='left',
                 name='time',
             ),
             method='nearest',
@@ -156,7 +185,7 @@ def impute_missing(data: pd.DataFrame, extrapolate=True):
         .transform(fillna)
     )
 
-    return data 
+    return data
 
 
 def nanint(x):
@@ -165,27 +194,26 @@ def nanint(x):
     return int(x)
 
 
-def read(filepath):
+def read(filepath, resample_hz='uniform'):
 
     p = pathlib.Path(filepath)
-    ftype = p.suffixes[0].upper()
+    ftype = p.suffixes[0].lower()
     fsize = round(p.stat().st_size / (1024 * 1024), 1)
 
-    if ftype in (".CSV", ".PKL"):
+    if ftype in (".csv", ".pkl"):
 
-        if ftype == ".CSV":
+        if ftype == ".csv":
             data = pd.read_csv(
                 filepath,
                 usecols=['time', 'x', 'y', 'z'],
                 parse_dates=['time'],
                 index_col='time'
             )
-        elif ftype == ".PKL":
+        elif ftype == ".pkl":
             data = pd.read_pickle(filepath)
         else:
             raise ValueError(f"Unknown file format: {ftype}")
 
-        # TODO: move this logic to actipy?
         freq = infer_freq(data.index)
         sample_rate = int(np.round(pd.Timedelta('1s') / freq))
 
@@ -194,7 +222,7 @@ def read(filepath):
             lowpass_hz=None,
             calibrate_gravity=True,
             detect_nonwear=True,
-            resample_hz='uniform',
+            resample_hz=resample_hz,
         )
 
         info = {
@@ -205,23 +233,26 @@ def read(filepath):
             **info
         }
 
-    elif ftype in (".CWA", ".GT3X", ".BIN"):
+    elif ftype in (".cwa", ".gt3x", ".bin"):
 
         data, info = actipy.read_device(
             filepath,
             lowpass_hz=None,
             calibrate_gravity=True,
             detect_nonwear=True,
-            resample_hz='uniform',
+            resample_hz=resample_hz,
         )
 
+    if 'ResampleRate' not in info:
+        info['ResampleRate'] = info['SampleRate']
+
     return data, info
 
 
 def infer_freq(x):
-    """ Like pd.infer_freq but more forgiving """ 
-    freq, _ = stats.mode(np.diff(x))
-    freq = pd.Timedelta(freq[0])
+    """ Like pd.infer_freq but more forgiving """
+    freq, _ = stats.mode(np.diff(x), keepdims=False)
+    freq = pd.Timedelta(freq)
     return freq
 
 
@@ -234,25 +265,25 @@ def resolve_path(path):
     return dirname, filename, extension
 
 
-def load_model(model_path=MODEL_PATH, force_download=False):
+def load_model(model_path, model_type, check_md5=True, force_download=False):
     """ Load trained model. Download if not exists. """
 
     pth = pathlib.Path(model_path)
 
     if force_download or not pth.exists():
 
-        # url = f"https://wearables-files.ndph.ox.ac.uk/files/models/stepcounter/{__model_version__}.joblib.lzma"
-        url = "https://tinyurl.com/34dswhss"  # 20220921.joblib.lzma
+        url = f"https://wearables-files.ndph.ox.ac.uk/files/models/stepcount/{__model_version__[model_type]}.joblib.lzma"
 
         print(f"Downloading {url}...")
 
         with urllib.request.urlopen(url) as f_src, open(pth, "wb") as f_dst:
             shutil.copyfileobj(f_src, f_dst)
 
-    assert md5(pth) == __model_md5__, (
-        "Model file is corrupted. Please run again with --force-download "
-        "to download the model file again."
-    )
+    if check_md5:
+        assert md5(pth) == __model_md5__[model_type], (
+            "Model file is corrupted. Please run with --force-download "
+            "to download the model file again."
+        )
 
     return joblib.load(pth)
 
